unit EZLabel;
// Don't you just hate the way it takes up time to get just the right size for
// a TLabel to fit in somewhere. Well, I got the solution - a autosizing
// Label that resizes the font to the best fit in the rectangle.
// Note: AutoSize must be false before it works
// Got this somewhere and just modified it (liked the 3d effect, so I used it)
// Component Editor and AFitType added by Louis Louw: louw@gcs.co.za

//Previous guy wrote:
//  I liked this 3dLabel so much, I decided to add an OnMouseOut event to it.
//  I don't know who originally wrote it, but OnMouseOut was added by
//  Durrin Hynes-Christensen: dxh@gv.dk

{
  This component is the 3rd generation of this component. I'm the third
  person to change or add something to this component. I added the
  component editor and the functionality of the text resizing to fit in
  the frame.
  This component is absolutely free and you can use it as you like.

  Please contact me if you have any bugs, comments, improvements, etc. at:
  e-mail: louw@gcs.co.za
  Snail-Mail:           P.O. Box 1138
                        Kroonstad
                        9500
                        South Africa
  Tel: (+27) (0562) 33234 (Talk to me live in person!)

  Please take the time to visit my webpage at www.gcs.co.za/mbs/mbs.htm
  for more great Delphi components and programming related stuff.

  Normal legal stuff:
  No one who contributed to writing this component can be held responsible
  for damages as a direct or indirect result of using this component. You may
  use this component at your own exclusive risk.
  Please do not sell this component. You got it free so please distribute it
  freely without any charge. Also please keep the above text and documentation
  with the distribution of the component.

 }
interface
{$R EZLabel.res}

uses Windows, Classes, Graphics, Forms, Controls,  Buttons, DsgnIntf,
   StdCtrls, ComCtrls, ExtCtrls, messages, Dialogs;


type
  T3DEffect = (Normal3d, Resit3d, Raised3d, Shadowed3d);
  TFitType = (NormalFit, BestFitVert, BestFithorz, BestFitBoth);


  TEZLabel = class(TLabel)
  private
    { Private declarations }

    procedure setStyleEffect(Value : T3DEffect);
    procedure SetShadowColor(Value:TColor);
    procedure SetWhiteColor(Value:TColor);
    procedure DoDrawText(var Rect: TRect; Flags: Word);
    procedure SetFhOffSet(value: integer);
    procedure SetFvOffSet(value: integer);
    procedure SetShadeLT(value: boolean);
    procedure SetFitType(Value: TFitType);
    procedure WMMouseMove(var msg: TWMMouseMove); message WM_MOUSEMOVE;
    procedure DoMouseOut(msg:TWMMouseMove);
    procedure DoMouseMove(msg:TWMMouseMove);
    procedure DoEdit;
    procedure UpdateDesigner;
    {SetAutoSize(Value : Boolean);}

  protected
    { Protected declarations }

    FOldWidth  : Integer;
    FOldHeight : Integer;
    FOldSize   : Integer;

    MouseOut:TNotifyEvent;
    MouseMove:TNotifyEvent;
    FFitType : TFitType;
    F3DEffect : T3DEffect;
    FShadowColor : TColor;
    FWhiteColor : TColor;
    FLast : TColor;
    FhOffSet,FvOffSet : integer;
    FShadeLTSet : boolean;
    procedure Paint; override;
    {procedure DoEdit;Virtual;}
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Align;
    property Caption;
    Property AFitType : TFitType read FFitType write SetFitType;

    property AStyle3D: T3DEffect read F3DEffect write setStyleEffect default Normal3d;
    property AShadeRightBottom: TColor read FShadowColor write SetShadowColor default clGray;
    property AShadeLeftTop: TColor read FWhiteColor write SetWhiteColor default clWhite;
    property AHShadeOffSet: integer read FhOffSet write SetFhOffSet default 5;
    property AVShadeOffSet: integer read FvOffSet write SetFvOffSet default -5;
    property AShadeLTSet: boolean read FShadeLTSet write setShadeLT default true;

    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property Visible;
    property Transparent;
    property Width;
    property Top;
    property Left;
    property Height;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove :TNotifyEvent read MouseMove write MouseMove;
    property OnMouseUp;
    property OnMouseOut:TNotifyEvent read MouseOut write MouseOut;
  end;

type
  TLabelEditorDlg = class(TForm)
    E3dStyle: TRadioGroup;
    EFitType: TRadioGroup;
    FontD: TFontDialog;
    btFont: TButton;
    Panel: TPanel;
    ECaption: TEdit;
    Label1: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    ColorD: TColorDialog;
    btColor: TButton;
    Button1: TButton;
    cTrans: TCheckBox;

    procedure OKButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure EFitTypeClick(Sender: TObject);
    procedure E3dStyleClick(Sender: TObject);
    procedure btFontClick(Sender: TObject);
    procedure ECaptionChange(Sender: TObject);
    procedure btColorClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure cTransClick(Sender: TObject);

  private
    EZLabel : TEZLabel;
  public
  end;


  { Component editor - brings up Label editor when double clicking on
    Labels property }
  TLabelEditor = class(TDefaultEditor)
  protected
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;


procedure Register;

implementation

uses SysUtils;

{$R *.DFM}

{ TLabelEditorDlg }

procedure TEZLabel.UpdateDesigner;
var
  ParentForm: TCustomForm;
begin
  if (csDesigning in ComponentState) and 
    not (csUpdating in ComponentState) then
  begin
    ParentForm := GetParentForm(Self);
    if Assigned(ParentForm) and Assigned(ParentForm.Designer) then
      ParentForm.Designer.Modified;
  end;
end;

{ TLabelsProperty }

procedure TEZLabel.DoEdit;
var
  LabelEditor: TLabelEditorDlg;
begin
  LabelEditor := TLabelEditorDlg.Create(Application);
  try
    LabelEditor.EZLabel.Font.Assign(Font);
    LabelEditor.EZLabel.Caption:=Caption;
    LabelEditor.EZLabel.Color:=Color;
    LabelEditor.EZLabel.Transparent:=Transparent;
    LabelEditor.EZLabel.AlignMent:=AlignMent;
    LabelEditor.EZLabel.AStyle3D:=AStyle3D;
    LabelEditor.EZLabel.AFitType:=AFitType;
    LabelEditor.ECaption.Text:=Caption;
    LabelEditor.cTrans.Checked:=Transparent;

    With LabelEditor do
    Case LabelEditor.EZLabel.AFitType of
     BestFitBoth : EFitType.ItemIndex:=0;
     BestFitHorz : EFitType.ItemIndex:=1;
     BestFitVert : EFitType.ItemIndex:=2;
     NormalFit   : EFitType.ItemIndex:=3;
    end;

    With LabelEditor do
    Case LabelEditor.EZLabel.AStyle3D of
     Normal3d   : E3dStyle.ItemIndex:=0;
     Resit3d    : E3dStyle.ItemIndex:=1;
     Raised3d   : E3dStyle.ItemIndex:=2;
     Shadowed3d : E3dStyle.ItemIndex:=3;
    End;

    LabelEditor.ShowModal;
    IF LabelEditor.modalResult = mrOK then
     Begin
      Caption:=LabelEditor.EZLabel.Caption;
      Color:=LabelEditor.EZLabel.Color;
      Transparent:=LabelEditor.EZLabel.Transparent;
      Font.Assign(LabelEditor.EZLabel.Font);
      Alignment:=LabelEditor.EZLabel.AlignMent;
      AStyle3d:=LabelEditor.EZLabel.AStyle3D;
      AFitType:=LabelEditor.EZLabel.AFitType;
      UpdateDesigner;
     End;
     Invalidate;

  finally
    LabelEditor.Free;
  end;
end;


{TLabelEditor}
procedure TLabelEditor.Edit;
begin
  TEZLabel(Component).DoEdit;
end;

function TLabelEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

function TLabelEditor.GetVerb(Index: Integer): string;
begin
  if Index = 0 then
    Result := 'Edit EZLabel'
  else Result := '';
end;

procedure TLabelEditor.ExecuteVerb(Index: Integer);
begin
  if Index = 0 then
   Begin
     TEZLabel(Component).DoEdit;
   End;
end;


procedure TLabelEditorDlg.OKButtonClick(Sender: TObject);
begin
end;


////Component
constructor TEZLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  //ControlStyle := ControlStyle + [csDesignInteractive];
  Transparent := True;
  ParentColor := False;
  FShadowColor := clGray;
  FWhiteColor := clWhite;
  FhOffSet := 5;
  FvOffSet := -5;
  FLast := clWhite;
  Autosize:=False;
  FFitType:=BestFitBoth;
  FOldSize:=Canvas.Font.Size;
  FOldWidth:=Width;
  FOldHeight:=Height;
end;

destructor TEZLabel.Destroy;
{------------------------------------------------------------------------------}
begin
  inherited Destroy;

end;


{Procedure TEZLabel.DoEdit;
var
  LabelEditor: TLabelEditor;
begin
  LabelEditor := TLabelEditor.Create(Application);
  try
    LabelEditor.ShowModal;
  finally
    LabelEditor.Free;
  end;
end;}

 procedure TEZLabel.DoDrawText( var Rect : TRect; Flags : Word );
  var
    {Text       : array[ 0..255 ] of Char; }
    Text : PChar;
    Size: Byte;
    TmpRect    : TRect;
    UpperColor : TColor;
    LowerColor : TColor;
    AddMe      : Integer;
  begin
  Size := GetTextLen;       {Get length of string in Edit1}
  Inc(Size);                      {Add room for null character}
  GetMem(Text, Size);
  GetTextBuf(Text, Size);           {Creates Buffer dynamic variable}
  {GetTextBuf(Text, SizeOf(Text)); }
    if ( Flags and DT_CALCRECT <> 0) and
       ( ( Text[0] = #0 ) or ShowAccelChar and
         ( Text[0] = '&' ) and
         ( Text[1] = #0 ) ) then
      StrCopy(Text, ' ');

    if not ShowAccelChar then
      Flags := Flags or DT_NOPREFIX;
    Canvas.Font := Font;

    if F3DEffect = Resit3d then
    begin
      UpperColor := FShadowColor;
      LowerColor := FWhiteColor;
    end
    else
    begin
      UpperColor := FWhiteColor;
      LowerColor := FShadowColor;
    end;

    if F3DEffect in [ Resit3d, Raised3d ] then
    begin
      TmpRect := Rect;
      OffsetRect( TmpRect, 1, 1 );
      Canvas.Font.Color := LowerColor;
      DrawText(Canvas.Handle, Text, StrLen(Text), TmpRect, Flags);

      TmpRect := Rect;
      OffsetRect( TmpRect, -1, -1 );
      Canvas.Font.Color := UpperColor;
      DrawText(Canvas.Handle, Text, StrLen(Text), TmpRect, Flags);
    end
    else if F3DEffect = Shadowed3d then
    begin
      TmpRect := Rect;
      OffsetRect( TmpRect, FhOffSet, FvOffSet );
      Canvas.Font.Color := LowerColor;
      DrawText(Canvas.Handle, Text, StrLen(Text), TmpRect, Flags);
    end;

    Canvas.Font.Color := Font.Color;

   If Not AutoSize then
   IF (FOldSize<>Canvas.Font.Size) or (FOldWidth<>Width) or (FOldHeight<>Height) then
    Begin
     Case FFitType of
       BestFitBoth : Begin
                       Canvas.Font.Size:=0;
                     While (abs(Canvas.font.height) < Rect.Bottom) and (Canvas.textwidth(String(text)) < Rect.Right) do
                      canvas.font.size:=canvas.font.size+1;
                      canvas.font.size:=canvas.font.size-1;
                      Font.Size:=Canvas.Font.Size;
                 End;
       BestFitVert : Begin
                       Font.Height:=Rect.Bottom;
                     End;
       BestFitHorz : Begin
                     Canvas.Font.Size:=0;
                     While (Canvas.textwidth(String(text)) < Rect.Right) do
                      canvas.font.size:=canvas.font.size+1;
                      canvas.font.size:=canvas.font.size-1;
                      Font.Size:=Canvas.Font.Size;
                 End;

     End;
     FOldSize:=Canvas.Font.Size;
     FOldWidth:=Width;
     FOldHeight:=Height;
    end;

    if not Enabled then
      Canvas.Font.Color := clGrayText;
    DrawText(Canvas.Handle, Text, StrLen(Text), Rect, Flags);
    //canvas.rectangle(rect.left,rect.top,rect.right,rect.bottom);
    FreeMem(Text, Size);
  end;


  procedure TEZLabel.Paint;
  const
    Alignments: array[TAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
  var
    Rect: TRect;
  begin
    with Canvas do
    begin
      if not Transparent then
      begin
        Brush.Color := Self.Color;
        Brush.Style := bsSolid;
        FillRect(ClientRect);
      end;
      Brush.Style := bsClear;
      Rect := ClientRect;
      DoDrawText( Rect, ( DT_EXPANDTABS or DT_WORDBREAK ) or
                  Alignments[ Alignment ] );
    end;
  end;

procedure TEZLabel.SetShadowColor(value: TColor);
begin
  if not (FShadowColor = value) then
  begin
     FShadowColor := value;
     invalidate;
   end;
 end;

Procedure TEZLabel.SetFitType(Value : TFitType);
Begin
 If Not (FFitType=Value) then
  Begin
    FFitType:=Value;
    FOldSize:=FOldSize+1;
    invalidate;
  end;
End;

procedure TEZLabel.SetFhOffSet(value: integer);
begin
  if value<>FhOffSet then
  begin
    FhOffSet := value;
    invalidate;
  end;
end;

procedure TEZLabel.SetFvOffSet(value: integer);
begin
  if value<>FvOffSet then
  begin
    FvOffSet := value;
    invalidate;
  end;
end;

procedure TEZLabel.SetWhiteColor(value: TColor);
begin
  if not (FWhiteColor = value) and (FShadeLTSet=false) then
  begin
     FWhiteColor := value;
     invalidate;
   end;
 end;

 procedure TEZLabel.setStyleEffect(value: T3DEffect);
 begin
   if F3DEffect <> value then
   begin
     F3DEffect := value;
     invalidate;
   end;
 end;

procedure TEZLabel.SetShadeLT(value: boolean);
begin
  if  FShadeLTSet <> value then
  begin
    FShadeLTSet := value;
    if FShadeLTSet = true then
    begin
      FLast := FWhiteColor;
      FWhiteColor := clWhite;
    end
    else
      FWhiteColor := Flast;
   end;
  invalidate;
end;

procedure TEZLabel.WMMouseMove(var msg: TWMMouseMove);
begin
  if not MouseCapture then
    SetCaptureControl(Self);
  if MouseCapture and ((msg.XPos < 0) or (msg.YPos < 0) or (msg.XPos > Width) or
     (msg.YPos > Height)) then {MouseOut}
  begin
    SetCaptureControl(nil);
    DoMouseOut(msg);
  end else DoMouseMove(msg);
end;

procedure TEZLabel.DoMouseOut(msg:TWMMouseMove);
begin
  if assigned(MouseOut) then MouseOut(self);
end;
procedure TEZLabel.DoMouseMove(msg:TWMMouseMove);
begin
  if assigned(MouseMove) then MouseMove(self);
end;

procedure TLabelEditorDlg.FormCreate(Sender: TObject);
begin
  EZLabel:=TEZLabel.Create(Self);
  With EZLabel Do
   Begin
     Parent:=Panel;
     Align:=AlClient;
     {Top:=Dummy.Top;
     Left:=Dummy.Left;
     Width:=Dummy.Width;
     Height:=Dummy.Height;}
     Caption:='Example';
   End;
end;

procedure TLabelEditorDlg.EFitTypeClick(Sender: TObject);
begin
 Case EFitType.ItemIndex of
   0 : EZLabel.AFitType:=BestFitBoth;
   1 : EZLabel.AFitType:=BestFitHorz;
   2 : EZLabel.AFitType:=BestFitVert;
   3 : EZLabel.AFitType:=NormalFit;
 end;
end;




procedure TLabelEditorDlg.E3dStyleClick(Sender: TObject);
begin
 Case E3dStyle.ItemIndex of
   0 : EZLabel.AStyle3D:=Normal3d;
   1 : EZLabel.AStyle3D:=Resit3d;
   2 : EZLabel.AStyle3D:=Raised3d;
   3 : EZLabel.AStyle3D:=Shadowed3d;
 End;


end;

procedure TLabelEditorDlg.btFontClick(Sender: TObject);
begin
  FontD.Font.Assign(EZLabel.Font);
   IF FontD.Execute then
     EZLabel.Font.Assign(FontD.Font);
end;

procedure TLabelEditorDlg.ECaptionChange(Sender: TObject);
begin
  EZLabel.Caption:=ECaption.Text;
end;

procedure Register;
begin
  RegisterComponents('MBS',[TEZLabel]);
  RegisterComponentEditor(TEZLabel, TLabelEditor);
  {RegisterPropertyEditor(TypeInfo(TLabels), nil, '', TLabelsProperty);}
end;

procedure TLabelEditorDlg.btColorClick(Sender: TObject);
begin
 ColorD.Color:=EZLabel.Font.Color;
 IF ColorD.Execute then
   EZLabel.Font.Color:=ColorD.Color;
end;

procedure TLabelEditorDlg.Button1Click(Sender: TObject);
begin
 ColorD.Color:=EZLabel.Color;
 IF ColorD.Execute then
  Begin
   EZLabel.Color:=ColorD.Color;
   EZLabel.Transparent:=False;
  end;
 cTrans.Checked:=EZLabel.Transparent; 
end;

procedure TLabelEditorDlg.cTransClick(Sender: TObject);
begin
 EZLabel.Transparent:=cTrans.Checked;
end;

end.
